<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Goods\Controller;

use Admin\Controller\AdminActionController;
use Admin\Data\Common;
use Doctrine\ORM\EntityManager;
use Goods\Entity\Goods;
use Goods\Entity\GoodsClass;
use Goods\Entity\GoodsClassContent;
use Goods\Entity\GoodsInClass;
use Goods\Form\GoodsClassForm;
use Goods\Service\GoodsBrandInClassManager;
use Goods\Service\GoodsClassContentManager;
use Goods\Service\GoodsClassManager;
use Goods\Service\GoodsInClassManager;
use Laminas\Mvc\I18n\Translator;
use Laminas\View\Model\JsonModel;
use Laminas\View\Model\ViewModel;

class GoodsClassController extends AdminActionController
{
    private $translator;
    private $entityManager;
    private $goodsClassManager;
    private $goodsInClassManager;
    private $goodsBrandInClassManager;
    private $goodsClassContentManager;

    public function __construct(
        Translator $translator,
        EntityManager $entityManager,
        GoodsClassManager $goodsClassManager,
        GoodsInClassManager $goodsInClassManager,
        GoodsBrandInClassManager $goodsBrandInClassManager,
        GoodsClassContentManager $goodsClassContentManager
    )
    {
        $this->translator       = $translator;
        $this->entityManager    = $entityManager;
        $this->goodsClassManager= $goodsClassManager;
        $this->goodsInClassManager = $goodsInClassManager;
        $this->goodsBrandInClassManager = $goodsBrandInClassManager;
        $this->goodsClassContentManager = $goodsClassContentManager;
    }

    /**
     * 商品分类列表
     * @return array|\Laminas\View\Model\ViewModel
     */
    public function indexAction()
    {
        $goodsClass = $this->entityManager->getRepository(GoodsClass::class)->findClassList();
        $classList  = Common::adminGoodsClass($goodsClass);

        return ['class' => $classList];
    }

    /**
     * 添加商品分类
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addAction()
    {
        $form = new GoodsClassForm();
        $form->get('classTopId')->setValueOptions($this->goodsPlugin()->classListOptions());
        $form->get('classTagGroup')->setValueOptions($this->goodsPlugin()->tagGroupOptions('', false));

        $response   = $this->getEventManager()->trigger('goods-class.add.show', $this, ['form' => $form, 'goodsClassType' => 'add']);
        $extend     = $response->last();
        if (isset($extend['form'])) $form = $extend['form'];

        if($this->getRequest()->isPost()) {
            $data = array_merge_recursive(
                $this->getRequest()->getPost()->toArray(),
                $this->getRequest()->getFiles()->toArray()
            );
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                try {
                    $classInfo = $this->goodsClassManager->addGoodsClass($data);
                    $this->goodsClassContentManager->addOrEditGoodsClassContent($data, $classInfo->getClassId());

                    $this->getEventManager()->trigger('goods-class.add.post', $this, ['data' => $data]);

                    $this->adminCommon()->addOperLog(sprintf($this->translator->translate('分类 %s 添加成功!'), $data['className'.SHOP_FONT_LANGUAGE]), $this->translator->translate('商品分类'));
                } catch (\Exception $e) {
                    $this->flashMessenger()->addWarningMessage($this->translator->translate('分类添加成功!'));
                }

                return $this->redirect()->toRoute('goods-class');
            }
        }

        return ['form' => $form, 'extend' => $extend];
    }

    /**
     * 编辑商品分类
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editAction()
    {
        $classId = (int) $this->params()->fromRoute('id', -1);

        $classInfo = $this->entityManager->getRepository(GoodsClass::class)->findOneByClassId($classId);
        if($classInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该分类不存在!'));
            return $this->redirect()->toRoute('goods-class');
        }

        $form = new GoodsClassForm($this->entityManager, $classInfo);
        $form->get('classTopId')->setValueOptions($this->goodsPlugin()->classListOptions());
        $form->get('classTagGroup')->setValueOptions($this->goodsPlugin()->tagGroupOptions('', false));

        $response   = $this->getEventManager()->trigger('goods-class.edit.show', $this, ['form' => $form, 'goodsClassType' => 'edit', 'classId' => $classId]);
        $extend     = $response->last();
        if (isset($extend['form'])) $form = $extend['form'];

        if($this->getRequest()->isPost()) {
            $data = array_merge_recursive(
                $this->getRequest()->getPost()->toArray(),
                $this->getRequest()->getFiles()->toArray()
            );
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                try {
                    $classImage = $classInfo->getClassImage();
                    $this->goodsClassManager->editGoodsClass($data, $classInfo);
                    $this->goodsClassContentManager->addOrEditGoodsClassContent($data, $classId);

                    $this->getEventManager()->trigger('goods-class.edit.post', $this, ['data' => $data, 'classImage' => $classImage, 'classInfo' => $classInfo]);

                    $this->adminCommon()->addOperLog(sprintf($this->translator->translate('分类 %s 编辑成功!'), $data['className'.SHOP_FONT_LANGUAGE]), $this->translator->translate('商品分类'));
                } catch (\Exception $e) {
                    $this->flashMessenger()->addWarningMessage($this->translator->translate('分类编辑成功!'));
                }

                return $this->redirect()->toRoute('goods-class');
            }
        } else {
            $valueArray = $classInfo->valuesArray();
            $valueArray['classTagGroup'] = unserialize($valueArray['classTagGroup']);
            $form->setData($valueArray);

            $classContent = $this->entityManager->getRepository(GoodsClassContent::class)->findBy(['classId' => $classId]);
            foreach ($classContent as $contentValue) {
                if (!in_array($contentValue->getLanguageCode(), SHOP_LANGUAGE)) continue;
                $form->setData([
                    'className'.$contentValue->getLanguageCode()        => $contentValue->getClassName(),
                    'classInfo'.$contentValue->getLanguageCode()        => $contentValue->getClassInfo(),
                    'classExtend'.$contentValue->getLanguageCode()      => $contentValue->getClassExtend(),
                    'classKeywords'.$contentValue->getLanguageCode()    => $contentValue->getClassKeywords(),
                    'classDescription'.$contentValue->getLanguageCode() => $contentValue->getClassDescription()
                ]);
            }
        }

        $query      = $this->entityManager->getRepository(GoodsInClass::class)->findClassInGoods(['class_id' => $classId]);
        $paginator  = $this->adminCommon()->shopPaginator($query, 1);

        return ['form' => $form, 'classInfo' => $classInfo, 'goodsList' => $paginator, 'extend' => $extend];
    }

    /**
     * 删除商品分类
     * @return mixed
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->adminCommon()->toReferer();

        $classId = (int) $this->params()->fromRoute('id', -1);

        $classInfo = $this->entityManager->getRepository(GoodsClass::class)->findOneByClassId($classId);
        if($classInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该分类不存在!'));
            return $this->adminCommon()->toReferer();
        }

        $subClassInfo = $this->entityManager->getRepository(GoodsClass::class)->findOneBy(['classTopId' => $classId]);
        if($subClassInfo) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该分类下存在分类，不能删除!'));
            return $this->adminCommon()->toReferer();
        }

        $goodsInfo = $this->entityManager->getRepository(Goods::class)->findOneBy(['classId' => $classId]);
        if($goodsInfo) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('商品里有该分类作为主分类的商品存在，不能删除!'));
            return $this->adminCommon()->toReferer();
        }

        $classContent = $this->entityManager->getRepository(GoodsClassContent::class)->findOneBy(['classId' => $classId, 'languageCode' => SHOP_FONT_LANGUAGE]);
        $this->adminCommon()->deleteImage($classInfo->getClassImage());
        $this->goodsClassManager->deleteGoodsClass($classInfo);
        $this->goodsInClassManager->delClassGoods($classId);
        $this->goodsBrandInClassManager->delGoodsBrandInClassId($classId);
        $this->goodsClassContentManager->deleteGoodsClassContent($classId);

        $this->getEventManager()->trigger('goods-class.delete.post', $this, ['classInfo' => $classInfo]);

        $this->adminCommon()->addOperLog(sprintf($this->translator->translate('分类 %s 删除成功!'), $classContent->getClassName()), $this->translator->translate('商品分类'));

        return $this->adminCommon()->toReferer();
    }

    /**
     * 批量处理排序
     * @return mixed
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function updateAllSortAction()
    {
        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            if(!empty($data['select_id'])) {
                $this->goodsClassManager->updateAllClassSort($data);

                $this->adminCommon()->addOperLog($this->translator->translate('批量排序处理成功!'), $this->translator->translate('商品分类'));
            }
        }

        return $this->adminCommon()->toReferer();
    }

    /**
     * 分类商品的分页ajax显示
     * @return ViewModel
     */
    public function ajaxClassGoodsAction()
    {
        $view = new ViewModel();
        $view->setTerminal(true);

        $classId = (int) $this->params()->fromRoute('id', -1);
        $page = (int) $this->params()->fromQuery('page', 1);
        $query      = $this->entityManager->getRepository(GoodsInClass::class)->findClassInGoods(['class_id' => $classId]);
        $paginator  = $this->adminCommon()->shopPaginator($query, $page);

        return $view->setVariables(['classId' => $classId, 'goodsList' => $paginator]);
    }

    /**
     * 更新单个分类商品信息
     * @return JsonModel
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function updateClassGoodsAction()
    {
        $message= $this->translator->translate('更新失败!');
        $inId   = (int) $this->request->getPost('inId', 0);
        $classGoodsSort = (int) $this->request->getPost('classGoodsSort', 1);
        $classGoodsRecommend = in_array($this->request->getPost('classGoodsRecommend'), [0, 1]) ? $this->request->getPost('classGoodsRecommend') : 0;

        $inClassGoods = $this->entityManager->getRepository(GoodsInClass::class)->findOneByInId($inId);
        if($inClassGoods) {
            $this->goodsInClassManager->updateInClassGoods(['classGoodsSort' => $classGoodsSort, 'classGoodsRecommend' => $classGoodsRecommend], $inClassGoods);
            $message = $this->translator->translate('更新成功!');
        }
        return new JsonModel(['msg' => $message]);
    }

    /**
     * 删除商品分类中的商品
     * @return JsonModel
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteClassGoodsAction()
    {
        $inId = (int) $this->params()->fromRoute('id', -1);
        $inClassGoods = $this->entityManager->getRepository(GoodsInClass::class)->findOneByInId($inId);
        if($inClassGoods == null) return new JsonModel(['msg' => $this->translator->translate('该分类商品不存')]);

        //当其为主分类商品时，不能删除
        $goodsInfo = $this->entityManager->getRepository(Goods::class)->findOneBy(['goodsId' => $inClassGoods->getGoodsId(), 'classId' => $inClassGoods->getClassId()]);
        if($goodsInfo != null) return new JsonModel(['msg' => $this->translator->translate('不能在此进行主分类商品删除，请到编辑商品中调整主分类后，再次删除!')]);

        $this->goodsInClassManager->delInClassGoods($inClassGoods);
        return new JsonModel(['state' => 'ok']);
    }
}